package com.sun.imageio.plugins.common;

import java.io.PrintStream;
import java.io.IOException;
import javax.imageio.stream.ImageOutputStream;

public class LZWCompressor
{
    int codeSize;
    int clearCode;
    int endOfInfo;
    int numBits;
    int limit;
    short prefix;
    BitFile bf;
    LZWStringTable lzss;
    boolean tiffFudge;
    
    public LZWCompressor(ImageOutputStream imageOutputStream, int codeSize, boolean tiffFudge) throws IOException {
        this.bf = new BitFile(imageOutputStream, !tiffFudge);
        this.codeSize = codeSize;
        this.tiffFudge = tiffFudge;
        this.clearCode = 1 << codeSize;
        this.endOfInfo = this.clearCode + 1;
        this.numBits = codeSize + 1;
        this.limit = (1 << this.numBits) - 1;
        if (this.tiffFudge) {
            --this.limit;
        }
        this.prefix = -1;
        (this.lzss = new LZWStringTable()).clearTable(codeSize);
        this.bf.writeBits(this.clearCode, this.numBits);
    }
    
    public void compress(byte[] array, int i, int n) throws IOException {
        int n2;
        int n3;
        int charString;
        for (n2 = i + n, ; i < n2; ++i) {
            n3 = array[i];
            if ((charString = this.lzss.findCharString(this.prefix, (byte)n3)) != -1) {
                this.prefix = charString;
            }
            else {
                this.bf.writeBits((int)this.prefix, this.numBits);
                if (this.lzss.addCharString(this.prefix, (byte)n3) > this.limit) {
                    if (this.numBits == 12) {
                        this.bf.writeBits(this.clearCode, this.numBits);
                        this.lzss.clearTable(this.codeSize);
                        this.numBits = this.codeSize + 1;
                    }
                    else {
                        ++this.numBits;
                    }
                    this.limit = (1 << this.numBits) - 1;
                    if (this.tiffFudge) {
                        --this.limit;
                    }
                }
                this.prefix = (short)((short)n3 & 0xFF);
            }
        }
    }
    
    public void flush() throws IOException {
        if (this.prefix != -1) {
            this.bf.writeBits((int)this.prefix, this.numBits);
        }
        this.bf.writeBits(this.endOfInfo, this.numBits);
        this.bf.flush();
    }
    
    public void dump(PrintStream printStream) {
        this.lzss.dump(printStream);
    }
}
